#Ce script a été écrit par Othmane Meskine et Solal Thomas
#Reconnaisance de formes et de couleurs pour un convoyeur de pièces

import numpy as np    #bibliothèque mathématique
import cv2            #bibliothèque de traitement d'images
import os             #bibliothèque fonctionnalité de gestion de fichier



#création du chemin jusqu'au dossier contenant les images à traiter
path=r"C:\Users\Admin\Documents\Python\\"
directory=r"C:\Users\Admin\Documents\Python"
os.chdir(directory)


#Dictionnaire associant couleur et position angulaire dans le cercle chromatique
colors={"Rouge1":[0,20],
        "Orange":[20,40],
        "Jaune":[40,80],
        "Vert":[80,160],
        "Cyan":[160,200],
        "Bleu":[200,260],
        "Magenta":[260,330],
        "Rouge":[330,360]
        }



#fonction de detéction de la couleur
#asssocie une couleur en moyennant la position dans le cercle chromatique les pixels compris dans un contours.
#entrée : 'image' est l'image que l'on veut analyser , 'cnt' est un contour d'une forme contenu dans la liste des contours 'contours' , 'colors' est le dictionnaire des couleurs
#sortie : 'color' la couleur détécté dans le contour 'cnt'

def detect_couleur(image,cnt,colors):
    image_hsv=cv2.cvtColor(image,cv2.COLOR_BGR2HSV)         #convertit l'image RGB en HSV
    mask = np.zeros(image_hsv.shape[:2], dtype="uint8")     #image noire servant de masque
    cv2.drawContours(mask, [cnt], -1, 255, -1)              #trace les contours sur le masque en remplissant de blanc
    mask = cv2.erode(mask, None, iterations=2)              #érosion du masque
    mean = cv2.mean(image_hsv, mask=mask)                   #Moyenne des couleurs des pixels contenues dans le contour
    if image.dtype=='uint8':                                #comparaison avec les valeurs du dictionnaires des couleurs (si sur 8bits)
        for color in colors:
            if int(2*mean[0]) in range(colors[color][0],colors[color][1]):
                return(color)
    else:                                                   #comparaison avec les valeurs du dictionnaires des couleurs (si non codé sur 8bits)
        for color in colors:
            if int(mean[0]) in range(colors[color][0],colors[color][1]):
                return(color)





#fonction de detéction
#recherche dans une image les formes géométriques simples et leurs couleurs
#entrée : 'Nom_Img' est le nom de l'image que l'on veut analyser au format BMP (avec le .bmp)
#sortie : 'NbForme' est un nombre entier du nombre de forme sur l'image, 'Listshape' liste des formes détéctées, 'Listcolor' liste des couleurs associées aux formes

def reco(Nom_Img):
    Npath=path + Nom_Img           #ouverture de l'image via son chemin d'accès
    image = cv2.imread(Npath)
    cv2.imshow('image',image)      #affichage de l'image
    cv2.waitKey(0)
    cv2.destroyAllWindows()

    gray = cv2.cvtColor(image, cv2.COLOR_BGR2GRAY)       #convertion en niveau de gris
    cv2.imshow('image',gray)                             #affichage de l'image en niveau de gris
    cv2.waitKey(0)
    cv2.destroyAllWindows()

    ret,thresh = cv2.threshold(gray,42,255,cv2.THRESH_BINARY)         #seuillage de l'image
    cv2.imshow('image',thresh)                                        # affichage du seuillage
    cv2.waitKey(0)
    cv2.destroyAllWindows()

    contours,h = cv2.findContours(thresh,cv2.RETR_EXTERNAL,cv2.CHAIN_APPROX_SIMPLE)   #détection des contours via la fct cv2.findcontours sur l'image seuillée, 'contours' contient les contours de chaque formes de l'image
    cv2.drawContours(image, contours, -1, (0,255,0), 3)                               #affichage de l'image avec les contours
    cv2.imshow('image',image)
    cv2.waitKey(0)
    cv2.destroyAllWindows()

    NbForme=0                  #definition des variables de sorties
    Listshape=[]
    Listcolor=[]

    for cnt in contours:                                        #pour chaque contour de forme détéctée
        perimetre=cv2.arcLength(cnt,True)                       #calcul du périmètre du contour


        if perimetre>1:                                         #vérification du périmètre non nul des artefact possible
            NbForme+=1
            approx = cv2.approxPolyDP(cnt,0.01*perimetre,True)  #approximation du contour (applatissement des cotés pour trouver les sommets)
            color = detect_couleur(image,cnt,colors)            #appel à la fct de détéction de la couleur
            M = cv2.moments(cnt)                                #recherche du centre du contour pour l'écriture dans l'affichage final
            cX = int(M["m10"] / (M["m00"]+1))
            cY = int(M["m01"] / (M["m00"]+1))
            cv2.drawContours(image,[cnt],-1,(0,255,0),2)        #dessine le contour sur l'image

            if len(approx)==3:                                  #recherche de la forme en comparant le nb de sommets
                shape = "triangle"
            elif len(approx)==4:
                (x, y, w, h) = cv2.boundingRect(approx)         #donne le rectangle associé au 4 sommets
                ratio = w / float(h)                            #donne le ratio hauteur/longueur du rectangle pour savoir si c'est un carré
                if ratio >= 0.95 and ratio <= 1.05:
                    shape = "carre"
                else:
                    shape = "rectangle"
            elif len(approx)==5:
                shape = "pentagone"
            elif len(approx)==6:
                shape = "hexagone"
            else:
                shape= "cercle"

            Listshape.append(shape)              #remplissage des variables de sorties
            Listcolor.append(color)

            cv2.putText(image,shape + " " + color, (cX, cY), cv2.FONT_HERSHEY_SIMPLEX,0.5, (255, 255, 255), 2)      #écrit sur l'image la forme et la couleur de la pièce

    cv2.imshow('image',image)                    #affichage de l'image avec contours + texte
    cv2.waitKey(0)
    cv2.destroyAllWindows()
    return NbForme, Listshape, Listcolor


print(reco("test3.bmp"))          #vérification des variables de sorties
